﻿//-----------------------------------------------------------------------
// <copyright file="MultiColorTextbox.cs" author="Artak Mkrtchyan">
//     Creative Commons - GNU General Public License (http://creativecommons.org/licenses/GPL/2.0/
// </copyright>
// <author>Artak Mkrtchyan</author>
// <email>mkArtak@gmail.com</email>
// <date>15/07/2012</date>
//-----------------------------------------------------------------------
using ConsoleHoster.Common.Utilities;
using ConsoleHoster.View.Utilities;
using ConsoleHoster.ViewModel;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Diagnostics;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;

namespace ConsoleHoster.View.Controls
{
	public class MultiColorTextbox : RichTextBox
	{
		private const string urlPattern = @"((ftp|https?)://|www.)[-a-zA-Z0-9@:%_\\\+.~#?&//=]+";

		public static readonly DependencyProperty TextProperty = DependencyProperty.Register("Text", typeof(ObservableCollection<OutputMessage>), typeof(MultiColorTextbox), new PropertyMetadata(null, new PropertyChangedCallback(OnTextPropertyChanged)));

		private FlowDocumentHelper documentHelper = null;
		private string textToFind = null;
		private int searchOccurances = 0;
		private bool callbackExpected = false;

		private static void OnTextPropertyChanged(DependencyObject argSender, DependencyPropertyChangedEventArgs argEA)
		{
			MultiColorTextbox tmpSender = argSender as MultiColorTextbox;

			tmpSender.Dispatcher.Invoke(new Action(() =>
			{
				ObservableCollection<OutputMessage> tmpOldValue = argEA.OldValue as ObservableCollection<OutputMessage>;
				ObservableCollection<OutputMessage> tmpNewValue = argEA.NewValue as ObservableCollection<OutputMessage>;
				if (tmpOldValue != tmpNewValue)
				{
					if (tmpOldValue != null)
					{
						tmpSender.UnRegisterTextChangeHandler(tmpOldValue);
					}

					tmpSender.AddTextInternal(tmpNewValue, true);

					if (tmpNewValue != null)
					{
						tmpSender.RegisterTextChangeHandler(tmpNewValue);
					}
				}
			}));
		}

		public MultiColorTextbox()
			: base()
		{
		}

		private void OnTextCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			if (Dispatcher.Thread != Thread.CurrentThread)
			{
				Dispatcher.Invoke(new Action(() => this.OnTextCollectionChanged(sender, e)));
				return;
			}

			if (this.callbackExpected)
			{
				this.callbackExpected = false;
				return;
			}

			switch (e.Action)
			{
				case NotifyCollectionChangedAction.Add:
					IList<OutputMessage> tmpList = e.NewItems.OfType<OutputMessage>().ToList();
					this.AddTextInternal(tmpList, false);
					break;

				case NotifyCollectionChangedAction.Move:
					break;

				case NotifyCollectionChangedAction.Remove:
					this.RemoveItemsInternal(e.OldItems.OfType<OutputMessage>().ToList());
					break;

				case NotifyCollectionChangedAction.Replace:
					break;

				case NotifyCollectionChangedAction.Reset:
					this.callbackExpected = true;
					this.Text.Clear();
					this.SingleParagraph.Inlines.Clear();
					break;
			}
		}

		private void AddTextInternal(IList<OutputMessage> items, bool argResetBeforAdd)
		{
			if (argResetBeforAdd)
			{
				this.SingleParagraph.Inlines.Clear();
			}

			if (items != null)
			{
				Paragraph tmpParagraph = this.SingleParagraph;

				int tmpCount = items.Count();
				for (int i = 0; i < tmpCount; i++)
				{
					OutputMessage tmpItem = items[i];

					if (Regex.IsMatch(tmpItem.Message, urlPattern))
					{
						MatchCollection tmpMatches = Regex.Matches(tmpItem.Message, urlPattern);

						int tmpLastPosition = 0;
						foreach (Match tmpMatch in tmpMatches)
						{
							if (tmpMatch.Index > tmpLastPosition)
							{
								Run tmpPart = new Run(tmpItem.Message.Substring(tmpLastPosition, tmpMatch.Index - tmpLastPosition));
								tmpPart.Foreground = new SolidColorBrush(tmpItem.Color);
								tmpParagraph.Inlines.Add(tmpPart);
							}

							Hyperlink tmpHyperlink = new Hyperlink(new Run(tmpMatch.Value));
							tmpHyperlink.NavigateUri = new Uri(tmpMatch.Value, UriKind.RelativeOrAbsolute);
							tmpHyperlink.Foreground = new SolidColorBrush(tmpItem.Color);
							tmpHyperlink.Cursor = Cursors.Hand;
							tmpHyperlink.ToolTip = "Use [Ctrl]+Click to navigate";
							tmpHyperlink.PreviewMouseLeftButtonUp += this.OnHyperlink_PreviewMouseLeftButtonUp;
							tmpParagraph.Inlines.Add(tmpHyperlink);

							tmpLastPosition = tmpMatch.Index + tmpMatch.Length;
						}

						if (tmpLastPosition < tmpItem.Message.Length)
						{
							Run tmpPart = new Run(tmpItem.Message.Substring(tmpLastPosition));
							tmpPart.Foreground = new SolidColorBrush(tmpItem.Color);
							tmpParagraph.Inlines.Add(tmpPart);
						}
					}
					else
					{
						tmpParagraph.Inlines.Add(GetRunObjectBoundToOutputMessage(tmpItem));
					}
				}
			}
		}

		private static Run GetRunObjectBoundToOutputMessage(OutputMessage argMessage)
		{
			Run tmpResult = new Run();
			Binding tmpTextBinding = new Binding
			{
				Path = new PropertyPath("Message"),
				Mode = BindingMode.OneWay,
				Source = argMessage
			};
			tmpResult.SetBinding(Run.TextProperty, tmpTextBinding);

			Binding tmpColorBinding = new Binding
			{
				Path = new PropertyPath("ColorBrush"),
				Mode = BindingMode.OneWay,
				Source = argMessage
			};
			tmpResult.SetBinding(Run.ForegroundProperty, tmpColorBinding);
			return tmpResult;
		}

		private void OnHyperlink_PreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
		{
			if (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl))
			{
				Hyperlink tmpLink = sender as Hyperlink;
				try
				{
					Process.Start(tmpLink.NavigateUri.ToString());
				}
				catch
				{
				}
			}
		}

		private void RemoveItemsInternal(IList<OutputMessage> argItemsToRemove)
		{
			foreach (OutputMessage tmpItem in argItemsToRemove)
			{
				this.RemoveMessageFromUI(tmpItem);
			}
		}

		private void RemoveMessageFromUI(OutputMessage tmpItem)
		{
			Run tmpRun = this.SingleParagraph.Inlines.OfType<Run>().Where(item => OutputMessageMatchesRun(tmpItem, item)).FirstOrDefault();
			if (tmpRun != null)
			{
				Inline tmpLB = this.SingleParagraph.Inlines.SkipWhile(item => item != tmpRun).Skip(1).Take(1).SingleOrDefault();
				this.SingleParagraph.Inlines.Remove(tmpRun);
				if (tmpLB != null && tmpLB is LineBreak)
				{
					this.SingleParagraph.Inlines.Remove(tmpLB);
				}
			}
		}

		private static bool OutputMessageMatchesRun(OutputMessage tmpItem, Run item)
		{
			return item.Text == tmpItem.Message && (item.Foreground as SolidColorBrush).Color == tmpItem.Color;
		}

		private void UnRegisterTextChangeHandler(ObservableCollection<OutputMessage> argCollection)
		{
			argCollection.CollectionChanged -= new NotifyCollectionChangedEventHandler(this.OnTextCollectionChanged);
		}

		private void RegisterTextChangeHandler(ObservableCollection<OutputMessage> argCollection)
		{
			argCollection.CollectionChanged += new NotifyCollectionChangedEventHandler(this.OnTextCollectionChanged);
		}

		public void FindNext(string argTextToFind)
		{
			if ((!String.IsNullOrEmpty(argTextToFind) && argTextToFind != this.TextToFind))
			{
				this.DocumentHelper.CurrentPosition = this.Document.ContentStart;
			}
			this.TextToFind = argTextToFind;

			if (!String.IsNullOrEmpty(this.TextToFind))
			{
				TextRange tmpRange = this.DocumentHelper.FindNext(argTextToFind, FindOptions.None);
				if (tmpRange != null)
				{
					this.Focus();
					this.Selection.Select(tmpRange.Start, tmpRange.End);
				}
				else
				{
					// Reached the end of the document, start from the top
					this.DocumentHelper.CurrentPosition = this.Document.ContentStart;
				}
			}
		}

		private int CountSearchOccurances()
		{
			int tmpResult = 0;
			this.DocumentHelper.CurrentPosition = this.Document.ContentStart;

			if (!String.IsNullOrEmpty(this.TextToFind))
			{
				TextRange tmpRange;
				while ((tmpRange = this.DocumentHelper.FindNext(this.TextToFind, FindOptions.None)) != null)
				{
					tmpResult++;
				}
			}

			return tmpResult;
		}

		public ObservableCollection<OutputMessage> Text
		{
			get
			{
				return this.GetValue(TextProperty) as ObservableCollection<OutputMessage>;
			}
			set
			{
				this.SetValue(TextProperty, value);
			}
		}

		private Paragraph SingleParagraph
		{
			get
			{
				return this.Document.Blocks.FirstBlock as Paragraph;
			}
		}

		public string TextToFind
		{
			get
			{
				return this.textToFind;
			}
			set
			{
				if (value != this.textToFind)
				{
					this.textToFind = value;
					this.DocumentHelper.CurrentPosition = this.Document.ContentStart;
					this.SearchOccurances = this.CountSearchOccurances();
				}
			}
		}

		private FlowDocumentHelper DocumentHelper
		{
			get
			{
				if (this.documentHelper == null)
				{
					this.documentHelper = new FlowDocumentHelper(this.Document);
				}
				return this.documentHelper;
			}
		}

		public int SearchOccurances
		{
			get
			{
				return this.searchOccurances;
			}

			private set
			{
				this.searchOccurances = value;
			}
		}
	}
}